function clickedButtonSendEdit() {
    let input = document.getElementById("correctionFileInput");
    addEditFile(input.files);
    // Reset the input after the click
    input.value = "";
    // To force the label update
    input.dispatchEvent(new Event("change"));
}

/**
 * Adds the file (the first / only one) from the file list to the viewer and the interface (drop-down lists)
 *
 * @param files A list of files but containing only one file (due to .nii or .nii.gz formats)
 */
function addEditFile(files) {
    // Chose an unique name
    let name = files[0].name;
    let postName = "";
    let i = 1;
    while (niiOptions.find((e) => e.image === name + postName) !== undefined) {
        postName = "_(" + i + ")";
        ++i;
    }
    name = name + postName;

    // add img parameters to the current container
    let imageParams = {};
    imageParams[name] = {
        name: name,
        category: "uploadedFiles",
    };

    if (
        papayaContainers[0].viewer.screenVolumes.length <=
        papaya.viewer.Viewer.MAX_OVERLAYS
    ) {
        papayaContainers[0].params = $.extend(
            {},
            papayaContainers[0].params,
            imageParams
        );
        papayaContainers[0].params["images"].push(name);
    }
    // add to viewer
    papayaContainers[0].viewer.loadImage(files, false, false, false);

    if (
        papayaContainers[0].viewer.screenVolumes.length >
        papaya.viewer.Viewer.MAX_OVERLAYS
    )
        return; // After the loadImage to keep the error message displayed
    papayaContainers[0].loadingImageIndex += 1;

    // update options to make the interface work properly
    niiOptions.push({
        image: name,
        options: imageParams[name],
    });

    // add to drop-down lists
    let selects = document.getElementsByName("uploadedFiles");
    selects.forEach(function (select) {
        let opt = document.createElement("option");
        opt.value = name;
        opt.id = name;
        opt.innerHTML = name;
        select.appendChild(opt);

        select.hidden = false;
        select.style.visibility = "visible";
        select.value = name;
    });

    hideOthersUploadedFiles(name);
    hideCategoryOverlays("segmentation");
}

/**
 * Hides all other files of the same category as the given file
 *
 * @param currentFileName Name of the file not to be hidden
 */
function hideOthersUploadedFiles(currentFileName) {
    const layoutOption = niiOptions.find((e) => e.image === currentFileName);
    const id = niiOptions.indexOf(layoutOption);
    for (let index = 0; index < niiOptions.length; index++) {
        if (
            index !== id &&
            "uploadedFiles" === niiOptions[index].options.category
        ) {
            papaya.Container.hideImage(0, index);
        }
    }
}

/**
 * Hides all overlays of a given category
 *
 * @param category Category of files to hide
 */
function hideCategoryOverlays(category) {
    for (let index = 0; index < niiOptions.length; index++) {
        if (category === niiOptions[index].options.category) {
            papaya.Container.hideImage(0, index);
        }
    }

    document.getElementsByName(category).forEach(function (elem) {
        elem.value = category;
    });
}
