//Constants
const NB_PLOT_POINTS = 91;
const CSV_LABEL_NAME_COL = 0;
const PLOT_BACKGROUND_COLOR = "rgba(180, 255, 200, 0.2)";
const PLOT_AVERAGE_BORDER_COLOR = "rgb(99, 255, 132)";
const PLOT_BOUNDS_BORDER_COLOR = "rgba(110, 210, 230, 1)";

//Variables
let PATIENT_POINT_COLOR = "rgb(255, 255, 255)";
let currChart;

function boundsCsvToArray(txt) {
    const rows = txt.split("\n");
    const arr = [];
    for (let index = 0; index < rows.length; index++) {
        const tokens = rows[index].split(";");
        arr[index] = tokens;
    }
    return arr;
}

function drawPlot(lowerBound, average, upperBound, point) {
    const xValues = [];
    for (let index = 0; index < NB_PLOT_POINTS; index++) {
        xValues.push(index);
    }
    const data = {
        labels: xValues,
        datasets: [
            {
                label: "point",
                backgroundColor: PATIENT_POINT_COLOR,
                borderColor: PATIENT_POINT_COLOR,
                data: point,
                pointRadius: 3,
            },
            {
                label: "min",
                backgroundColor: PLOT_BACKGROUND_COLOR,
                borderColor: PLOT_BOUNDS_BORDER_COLOR,
                data: lowerBound,
                tension: 0.3,
                pointRadius: 0,
                fill: "+2",
            },
            {
                label: "medium",
                backgroundColor: PLOT_BACKGROUND_COLOR,
                borderColor: PLOT_AVERAGE_BORDER_COLOR,
                data: average,
                tension: 0.3,
                pointRadius: 0,
                fill: false,
                legend: false,
            },
            {
                label: "max",
                backgroundColor: PLOT_BACKGROUND_COLOR,
                borderColor: PLOT_BOUNDS_BORDER_COLOR,
                data: upperBound,
                fill: false,
                pointRadius: 0,
                tension: 0.3,
            },
        ],
    };

    const config = {
        type: "line",
        data: data,
        options: {
            interaction: {
                mode: "index",
                intersect: false,
            },
            scales: {
                y: {
                    title: {
                        display: true,
                        text: "% ICC",
                    },
                },
                x: {
                    title: {
                        display: true,
                        text: "Age",
                    },
                },
            },
            plugins: {
                filler: {
                    propagate: false,
                },
                legend: {
                    labels: false,
                },
            },
        },
    };

    if (currChart) {
        currChart.destroy();
    }
    currChart = new Chart(document.getElementById("myChart"), config);
}

function updatePlotsData(
    arrData,
    lowerBoundArr,
    averageArr,
    upperBoundArr,
    point,
    patientData
) {
    for (let label_idx = 0; label_idx < arrData.length; label_idx++) {
        const labelToTest =
            arrData[label_idx][CSV_LABEL_NAME_COL].toLowerCase();

        if (isSameLabel(labelToTest, CURRENT_LABEL_NAME)) {
            //Re-creating average+bounds from the array
            for (let age_idx = 0; age_idx < NB_PLOT_POINTS; age_idx++) {
                lowerBoundArr[age_idx] = arrData[label_idx][age_idx * 3 + 1];
                averageArr[age_idx] = arrData[label_idx][age_idx * 3 + 2];
                upperBoundArr[age_idx] = arrData[label_idx][age_idx * 3 + 3];

                if (age_idx == patientData.age) {
                    point[age_idx] = patientData.structure_value;
                    if (
                        patientData.structure_value > upperBoundArr[age_idx] ||
                        patientData.structure_value < lowerBoundArr[age_idx]
                    ) {
                        PATIENT_POINT_COLOR = "rgb(255, 0, 0)";
                    } else {
                        PATIENT_POINT_COLOR = "rgb(0, 255, 0)";
                    }
                } else {
                    point[age_idx] = null;
                }
            }
            document.getElementById("labelPlot").hidden = false;
            break;
        } else {
            document.getElementById("labelPlot").hidden = true;
        }
    }
}

function isSameLabel(labelA, labelB) {
    if (labelA !== undefined && labelB !== undefined) {
        const labelAasArray = labelA
            .replace("_", " ")
            .toLowerCase()
            .split(" ")
            .sort();
        const labelBasArray = labelB
            .replace("_", " ")
            .toLowerCase()
            .split(" ")
            .sort();
        if (labelAasArray.join(",") == labelBasArray.join(",")) {
            return true;
        }
    }

    return false;
}

function reportCsvToJson(txt) {
    const rows = txt.split("\n");
    const def_row = rows[0].split(";");
    const value_row = rows[1].split(";");

    const res = {
        age: "",
        sex: "",
        structure_value: "",
        bounds_file: "",
    };

    if (def_row.length === value_row.length) {
        for (let index = 0; index < def_row.length; index++) {
            const key = def_row[index].toLowerCase();
            //Based on Volume % only
            if (key.includes("cm3") || key.includes("asymmetry")) {
                continue;
            }

            const value = value_row[index].toLowerCase();

            if (key === "age") {
                if (value > 100) {
                    res.age = 100;
                } else {
                    res.age = value;
                }
                continue;
            }

            if (key === "sex") {
                res.sex = value;
                continue;
            }

            const labelRemoveTotal = key.replace(" total", "");
            const labelToTest = labelRemoveTotal.replace(" volume %", "");

            if (isSameLabel(labelToTest, CURRENT_LABEL_NAME)) {
                res.structure_value = value;
                break;
            }
        }

        switch (res.sex) {
            case "male":
                res.bounds_file = bounds_files.male;
                break;

            case "female":
                res.bounds_file = bounds_files.female;
                break;

            default:
                res.bounds_file = bounds_files.general;
                break;
        }
    }

    return res;
}

async function updatePatientData() {
    //read the CSV from "job"+job_hash+"/report.csv"
    const report_path = "data/job" + job_hash + "/report.csv";

    const res = await fetch(report_path)
        .then((response) => response.text())
        .then((text) => {
            const patientData = reportCsvToJson(text);
            return patientData;
        })
        .catch((error) => {
            console.log(error);
            return;
        });

    return res;
}

async function displayPlot() {
    // Dont redraw a plot if it's the same label
    if (LABEL_PLOT !== CURRENT_LABEL) {
        LABEL_PLOT = CURRENT_LABEL;
        const patientData = await updatePatientData();
        if (!patientData) {
            return;
        }
        fetch("data/" + patientData.bounds_file)
            .then((response) => response.text())
            .then((text) => {
                const arrData = boundsCsvToArray(text);
                const lowerBoundArr = [];
                const averageArr = [];
                const upperBoundArr = [];
                const newPatientPoint = [];
                updatePlotsData(
                    arrData,
                    lowerBoundArr,
                    averageArr,
                    upperBoundArr,
                    newPatientPoint,
                    patientData
                );
                drawPlot(
                    lowerBoundArr,
                    averageArr,
                    upperBoundArr,
                    newPatientPoint
                );
            })
            .catch((error) => console.log(error));
    }
}
