//Constants
const IMAGE_QC_ISSUES = ["resolution", "noise", "contrast", "artefacts"];
const SEGMENTATION_QC_ISSUES = [
    "over_segmentation",
    "under_segmentation",
    "missing_structures",
];

//Variables
let CURRENT_IMAGE_LAYER_INDEX = "";
let CURRENT_SEGMENTATION_LAYER_INDEX = "";

function updateForm() {
    const layersToUpdate = [];

    //The first time the method is called
    //  => we set the index variables depending on the pre-selected layers (niiOptions)
    if (
        CURRENT_IMAGE_LAYER_INDEX == "" &&
        CURRENT_SEGMENTATION_LAYER_INDEX == ""
    ) {
        CURRENT_IMAGE_LAYER_INDEX = preEnabledLayers[0];
        CURRENT_SEGMENTATION_LAYER_INDEX = preEnabledLayers[1];
    }

    layersToUpdate.push(job_qc.layers[CURRENT_IMAGE_LAYER_INDEX]);
    layersToUpdate.push(job_qc.layers[CURRENT_SEGMENTATION_LAYER_INDEX]);
    layersToUpdate.forEach((layer) => {
        updateFormFields(layer);
    });
}

function displaySubForm(layer, text, issues) {
    let checkGood;
    let checkMedium;
    let checkBad;

    if (layer.category == "image") {
        checkGood = document.getElementById("checkGoodImage");
        checkMedium = document.getElementById("checkMediumImage");
        checkBad = document.getElementById("checkBadImage");
    } else if (layer.category == "segmentation") {
        checkGood = document.getElementById("checkGoodSegmentation");
        checkMedium = document.getElementById("checkMediumSegmentation");
        checkBad = document.getElementById("checkBadSegmentation");
    }

    switch (layer.mark) {
        case "Good":
            checkGood.checked = true;
            text.style.display = "Block";
            issues.style.display = "none";
            break;
        case "Medium":
            checkMedium.checked = true;
            text.style.display = "Block";
            issues.style.display = "Block";
            break;
        case "Bad":
            checkBad.checked = true;
            text.style.display = "Block";
            issues.style.display = "Block";
            break;
        default:
            break;
    }
}

function checkIssues(layer) {
    let issues = [];
    if (layer.category == "image") {
        issues = IMAGE_QC_ISSUES;
    } else if (layer.category == "segmentation") {
        issues = SEGMENTATION_QC_ISSUES;
    }

    issues.forEach((issueId) => {
        const issueCheckBox = document.getElementById(issueId);
        if (layer.issues.includes(issueId)) {
            issueCheckBox.checked = true;
        } else {
            issueCheckBox.checked = false;
        }
    });
}

function fillComment(layer, text) {
    //TODO: Change the comment section in the html so we don't have to find the children 'textArea'
    text.children[1].value = layer.comment;
}

function updateFormFields(layer) {
    let text = "";
    let issues = "";

    if (layer.category == "image") {
        text = document.getElementById("textImage");
        issues = document.getElementById("imageIssues");
    } else if (layer.category == "segmentation") {
        text = document.getElementById("textSegmentation");
        issues = document.getElementById("segmentationIssues");
    }

    displaySubForm(layer, text, issues);
    checkIssues(layer);
    fillComment(layer, text);
}
const GOOD_CHECK_TYPE = 0;
const MEDIUM_CHECK_TYPE = 1;
const BAD_CHECK_TYPE = 2;

//TODO: maybe use event.target.value like checkIssue() ?
function checkMark(type, isImage) {
    let CUR_LAYER;
    if (isImage) {
        CUR_LAYER = CURRENT_IMAGE_LAYER_INDEX;
    } else {
        CUR_LAYER = CURRENT_SEGMENTATION_LAYER_INDEX;
    }
    switch (type) {
        case GOOD_CHECK_TYPE:
            job_qc.layers[CUR_LAYER].mark = "Good";
            job_qc.layers[CUR_LAYER].issues = [];
            break;
        case MEDIUM_CHECK_TYPE:
            job_qc.layers[CUR_LAYER].mark = "Medium";
            break;
        case BAD_CHECK_TYPE:
            job_qc.layers[CUR_LAYER].mark = "Bad";
            break;
        default:
            job_qc.layers[CUR_LAYER].mark = "";
            break;
    }
    updateForm();
}

function checkIssue(event, isImage) {
    let CUR_LAYER;
    let issues = [];

    if (isImage) {
        CUR_LAYER = CURRENT_IMAGE_LAYER_INDEX;
        issues = IMAGE_QC_ISSUES;
    } else {
        CUR_LAYER = CURRENT_SEGMENTATION_LAYER_INDEX;
        issues = SEGMENTATION_QC_ISSUES;
    }

    const issue = event.target.value;
    //Making sure we add a known issue to job_qc
    if (!issues.includes(issue)) {
        return;
    }

    if (event.target.checked) {
        job_qc.layers[CUR_LAYER].issues.push(issue);
    } else {
        const index = job_qc.layers[CUR_LAYER].issues.indexOf(issue);
        if (index > -1) {
            job_qc.layers[CUR_LAYER].issues.splice(index, 1);
        }
    }
}

function updateComment(event, isImage) {
    let CUR_LAYER;
    if (event.target.value.length <= 280) {
        if (isImage) {
            CUR_LAYER = CURRENT_IMAGE_LAYER_INDEX;
        } else {
            CUR_LAYER = CURRENT_SEGMENTATION_LAYER_INDEX;
        }
        job_qc.layers[CUR_LAYER].comment = event.target.value;
    } else {
        return;
    }
}

function updateCommentButtons(needsUpdate) {
    const satisfiedButtons = document.getElementById("satisfiedButtons");
    if (needsUpdate) {
        document.getElementById("buttonCSVQC").hidden = false;
        satisfiedButtons.innerHTML =
            '<a href="#comments" data-toggle="tab" class="comments-link"><button type="button" class="btn btn-dark m-1">Edit</button></a>';
        $(".comments-link").click(function () {
            $("#myTabs li:nth-child(2) a").tab("show");
        });
    }
}

//POST : Permet de sauvegarder la valeur de notre variable job_qc dans l'input
async function sendFormResults() {
    const params = new URLSearchParams();
    params.append("job_qc", JSON.stringify(job_qc));
    const res = await axios.post("/save-input?hash=" + job_hash, params);
    if (res.data.updated) {
        document.getElementById("buttonCSVQC").hidden = false;
        updateCommentButtons(res.data.updated);
    }
}

function chooseEditFile(event) {
    if (event.target.files[0] !== undefined) {
        document.getElementById("buttonSendEdit").disabled = false;
    } else {
        document.getElementById("buttonSendEdit").disabled = true;
    }
}

//When changing layer from the QC tab => we want to update the fields too
function convertQCToCSV() {
    let str =
        "filename;layerName;mark;issues;issues;issues;issues;comment;category\n";
    for (let i = 0; i < job_qc.layers.length; i++) {
        let line = "";
        for (const index in job_qc.layers[i]) {
            const element = job_qc.layers[i][index];
            if (index === "issues") {
                for (let y = 0; y < 4; y++) {
                    line += ";";
                    if (y < element.length) {
                        line += element[y];
                    }
                }
            } else {
                if (line != "") {
                    line += ";";
                }
                line += element;
            }
        }
        str += line + "\r\n";
    }
    const hiddenElement = document.createElement("a");
    hiddenElement.href = "data:text/csv;charset=utf-8," + encodeURI(str);
    hiddenElement.target = "_blank";
    // Name of the file
    hiddenElement.download = "comments" + job_hash + ".csv";
    hiddenElement.click();
}

function changeLayer(obj, allFilesOptions) {
    toggleOverlay(obj, allFilesOptions);
    job_qc.layers.forEach((layer) => {
        if (layer.fileName == obj.value) {
            if (layer.category == "image") {
                CURRENT_IMAGE_LAYER_INDEX = job_qc.layers.indexOf(layer);
            } else if (layer.category == "segmentation") {
                CURRENT_SEGMENTATION_LAYER_INDEX = job_qc.layers.indexOf(layer);
            }
        }
    });
    updateForm();
}

//Sert à l'initialisation des menus si l'utilisateur n'a pas changé de calque
const preEnabledLayers = [];

niiOptions.forEach((optionLayer) => {
    if (optionLayer.options.enabled) {
        job_qc.layers.forEach((qc_layer) => {
            if (qc_layer.fileName == optionLayer.image) {
                const layerIdx = job_qc.layers.indexOf(qc_layer);
                if (layerIdx > -1) {
                    preEnabledLayers.push(layerIdx);
                }
            }
        });
    }
});

//Initialisation des menus déroulants de l'onglet QC
job_qc.layers.forEach((layer) => {
    if (layer.category == "image") {
        let imageQCSelect = document.getElementById("imageQC");
        let optionToAdd =
            "<option value='" +
            layer.fileName +
            "' id='" +
            layer.fileName +
            "QC'";
        if (preEnabledLayers.includes(job_qc.layers.indexOf(layer))) {
            optionToAdd += " selected ";
        }
        optionToAdd += ">" + layer.layerName + "</option>";
        imageQCSelect.innerHTML += optionToAdd;
    } else if (layer.category == "segmentation") {
        let segmentationQCSelect = document.getElementById("segmentationQC");
        let optionToAdd =
            "<option value='" +
            layer.fileName +
            "' id='" +
            layer.fileName +
            "QC'";
        if (preEnabledLayers.includes(job_qc.layers.indexOf(layer))) {
            optionToAdd += " selected ";
        }
        optionToAdd += ">" + layer.layerName + "</option>";
        segmentationQCSelect.innerHTML += optionToAdd;
    }
});

//=============Image Form Events================
//Mark Checks
document.getElementById("checkGoodImage").addEventListener(
    "click",
    () => {
        checkMark(GOOD_CHECK_TYPE, true);
    },
    false
);
document.getElementById("checkMediumImage").addEventListener(
    "click",
    () => {
        checkMark(MEDIUM_CHECK_TYPE, true);
    },
    false
);
document.getElementById("checkBadImage").addEventListener(
    "click",
    () => {
        checkMark(BAD_CHECK_TYPE, true);
    },
    false
);

//Image Issues
document.getElementById("resolution").addEventListener(
    "click",
    (event) => {
        checkIssue(event, true);
    },
    false
);
document.getElementById("noise").addEventListener(
    "click",
    (event) => {
        checkIssue(event, true);
    },
    false
);
document.getElementById("contrast").addEventListener(
    "click",
    (event) => {
        checkIssue(event, true);
    },
    false
);
document.getElementById("artefacts").addEventListener(
    "click",
    (event) => {
        checkIssue(event, true);
    },
    false
);

//Image Comment Section
document.getElementById("imageComment").addEventListener(
    "change",
    (event) => {
        updateComment(event, true);
    },
    false
);

//=============Segmentation Form Events================
//Segmentation Mark Checks
document.getElementById("checkGoodSegmentation").addEventListener(
    "click",
    () => {
        checkMark(GOOD_CHECK_TYPE, false);
    },
    false
);
document.getElementById("checkMediumSegmentation").addEventListener(
    "click",
    () => {
        checkMark(MEDIUM_CHECK_TYPE, false);
    },
    false
);
document.getElementById("checkBadSegmentation").addEventListener(
    "click",
    () => {
        checkMark(BAD_CHECK_TYPE, false);
    },
    false
);

//Segmentation Issues
document.getElementById("over_segmentation").addEventListener(
    "click",
    (event) => {
        checkIssue(event, false);
    },
    false
);
document.getElementById("under_segmentation").addEventListener(
    "click",
    (event) => {
        checkIssue(event, false);
    },
    false
);
document.getElementById("missing_structures").addEventListener(
    "click",
    (event) => {
        checkIssue(event, false);
    },
    false
);

//Segmentation Comment Section
document.getElementById("segmentationComment").addEventListener(
    "change",
    (event) => {
        updateComment(event, false);
    },
    false
);

// ToolTip on Buttons
$(function () {
    $('[data-toggle="tooltip"]').tooltip();
});

// Edition issues
document.getElementById("correctionFileInput").addEventListener(
    "change",
    (event) => {
        chooseEditFile(event);
    },
    false
);
