//Variables
let LABELS_descriptions = undefined;
let CURRENT_LABEL_NAME = ""; // Use in graph.js
let OLD_CURRENT_LABEL = ""
let CURRENT_LABEL_DESCRIPTION = "";
let LABEL_PLOT = ""; // Use in graph.js

function showLabel(filename) {
    // Get the label at the coord of the crossbar
    const xCross = papayaContainers[0].viewer.currentCoord.x;
    const yCross = papayaContainers[0].viewer.currentCoord.y;
    const zCross = papayaContainers[0].viewer.currentCoord.z;
    CURRENT_LABEL = papayaContainers[0].viewer.getCurrentValueAt(
        xCross,
        yCross,
        zCross
    );
    screenVol = papayaContainers[0].viewer.currentScreenVolume;
    const screenCat =
        screenVol.volume.params[screenVol.volume.fileName].category;
    // Hide informations for no Label, no Segmentation or an image is selected
    if (
        document.getElementById("segmentation").value == "segmentation" ||
        !CURRENT_LABEL ||
        screenCat === "image"
    ) {
        document.getElementById("labelDiv").hidden = true;
        return;
    }
    let mapping_path = "";
    //Getting mapping path from the input.json
    niiOptions.forEach((element) => {
        if (element.image == filename && element.options.regions_mapping) {
            mapping_path = element.options.regions_mapping;
        }
    });

    //Reading the mapping specified in the input (if exists/we find it)
    if (mapping_path !== "" && CURRENT_LABEL != OLD_CURRENT_LABEL) {
        //We can use Public folders/files directly
        fetch("data/" + mapping_path)
            .then((response) => {
                const mapping = response.json();
                return mapping;
            })
            .then((mapping) => {
                //Using the field mapping["LabelIdx"] to find the right label now from the right mapping_file
                OLD_CURRENT_LABEL = CURRENT_LABEL
                CURRENT_LABEL_NAME = mapping[CURRENT_LABEL.toString()];
                CURRENT_LABEL_DESCRIPTION =
                    LABELS_descriptions[CURRENT_LABEL_NAME] !== undefined
                        ? LABELS_descriptions[CURRENT_LABEL_NAME]
                        : "Element without description";
                if (CURRENT_LABEL_NAME === undefined) {
                    document.getElementById("labelDiv").hidden = true;
                } else {
                    document.getElementById("labelDiv").hidden = false;
                    document.getElementById("labelName").innerHTML =
                        "<h5>" + CURRENT_LABEL_NAME + "</h5>";
                    document.getElementById("labelDescription").innerHTML =
                        CURRENT_LABEL_DESCRIPTION;
                }
            })
            .catch((error) => {
                console.log(error);
            });
    }
}

function loadLabelsDescriptions(path) {
    if (LABELS_descriptions === undefined) {
        fetch(path).then(async (response) => {
            LABELS_descriptions = await response.json();
        });
    } else {
        document.getElementById("labelDiv").hidden = true;
    }
}

function toggleOverlay(obj, allFilesOptions) {
    const value = obj.value;
    const layoutOption = allFilesOptions.find((e) => e.image === value);
    const id = allFilesOptions.indexOf(layoutOption);
    for (let index = 0; index < allFilesOptions.length; index++) {
        if (index === id) {
            // Show and select the overlay
            papaya.Container.showImage(0, index);
            papayaContainers[0].viewer.setCurrentScreenVol(index);
            papayaContainers[0].toolbar.updateImageButtons();
        } else if (obj.name === allFilesOptions[index].options.category) {
            // Hide all other overlay of the category
            papaya.Container.hideImage(0, index);
        }
    }
    document.getElementsByName(obj.name).forEach(function (elem) {
        elem.value = obj.value;
    });
    // Redraw the information after the switch
    LABEL_PLOT = "";
    showLabel(papayaContainers[0].viewer.currentScreenVolume.volume.fileName);
    displayPlot();
}

loadLabelsDescriptions("data/labels_descriptions.json");

// LEUR FICHIER

const PreviewTablesConfigs = {
    last_jobs: {
        ajax: {
            url: "/user/api/v1.0/jobs",
            dataSrc: "",
        },
        pageLength: 15,
        lengthMenu: [15, 30],
        lengthChange: true,
        dom: "pt",
        autoWidth: true,
        processing: true,
        pagingType: "simple",
        columnDefs: [
            {
                targets: [0],
                visible: true,
                responsivePriority: 1,
                title: "Date",
                type: "str",
                data: "created",
                width: "40%",
                render: function (data, type, full, meta) {
                    var datetime = moment(data);
                    date = datetime.format("YYYY-MM-DD");
                    return type == "display"
                        ? '<span class="mr-auto">' + date + "</span>"
                        : data;
                },
            },
            {
                targets: [1],
                visible: true,
                responsivePriority: 1,
                title: "Pipeline",
                type: "str",
                data: "pipeline_name",
                width: "40%",
            },
            {
                targets: [2],
                responsivePriority: 2,
                visible: true,
                className: "text-right",
                title: "Actions",
                width: "20%",
                data: null,
                render: function (data, type, full, meta) {
                    if (type != "display") {
                        return null;
                    }
                    var html = [];
                    html.push(
                        '<a href="#" class="btn btn-primary editor_preview">'
                    );
                    html.push('<span class="fa  fa-eye"></span>');
                    html.push("</a>");
                    return html.join("");
                },
            },
        ],
        order: [[1, "desc"]],
        stateSave: true,
        stateSaveParams: function (settings, data) {
            data.search.search = "";
        },
        select: false,
        responsive: true,
    },
};

function updateUserJobsTable() {
    const table = $("#jobs_table").DataTable(PreviewTablesConfigs.last_jobs);

    $("#jobs_table tbody").on("click", "a.editor_preview", function (e) {
        var tr = $(this).closest("tr");
        var data = table.row(tr).data();
        window.location =
            "/user/preview?kiosk=True&hash=" + data["hash_result"];
    });
}

function downloadPreviewImages() {}

function renderPapayaViewer() {
    try {
        if (niiFiles.length < 1 && jobHash != null) {
            downloadPreviewImages();
        } else {
            papaya.Container.startPapaya();
        }
    } catch (e) {
        console.log("Papaya not started");
    }
}

$(document).ready(function () {
    renderPapayaViewer();
    //updateUserJobsTable();

    $(".comments-link").click(function () {
        $("#myTabs li:nth-child(2) a").tab("show");
    });

    $("#satisfied").click(function () {
        job_qc.layers.forEach((layer) => {
            layer.mark = "Good";
        });
        updateForm();
    });

    $("#notSatisfied").click(function () {
        job_qc.layers.forEach((layer) => {
            layer.mark = "Medium";
        });
        updateForm();
    });

    // Change the arrow on Description Button
    $("#collapseDescription").on("show.bs.collapse", function () {
        document.getElementById("descriptionarrow").className =
            "fas fa-chevron-up";
    });

    $("#collapseDescription").on("hide.bs.collapse", function () {
        document.getElementById("descriptionarrow").className =
            "fas fa-chevron-down";
    });

    updateCommentButtons(job_qc.commentExist);
    updateForm();
});
