const express = require("express");
const router = express.Router();
const fs = require('fs').promises;

router.get("/preview", async function (req, res) {

    const hash = req.query.hash;
    var inputPath = __dirname + "/../public/data/";
    if (hash) {
        inputPath += "job" + hash + "/";
    }
    inputPath += "input.json";

    //Retrieving input and initializing, if not already, the 'job_qc' field of input
    const input = await fs.readFile(inputPath).then(obj => {
        return JSON.parse(obj)
    }).then(data => {
        var input = data;
        //console.log(input);
        if (!input.job_qc || Object.keys(input.job_qc).length == 0) {
            var job_qc = {
                commentExist: false,
                layers: [],
            };

            input.preview_options.forEach(layer => {
                const fileName = layer.image;
                const layerName = layer.options.name;
                const layerQcSkeleton = {
                    fileName: fileName,
                    layerName: layerName,
                    mark: "",
                    issues: [],
                    comment: "",
                    category: "",
                }
                layerQcSkeleton.category = layer.options.category;
                job_qc.layers.push(layerQcSkeleton);
            });

            input.job_qc = job_qc;

            fs.writeFile(inputPath, JSON.stringify(input, null, 2), function (err) {
                if (err) throw err;
            });
        }

        return input;
    }).catch(err => {
        console.log(err)
    })

    if(!input){
        //Send error status
        res.status(500).send("No input found with this hash");
    }

    const nii_options = input.preview_options;

    var job_qc = input.job_qc;

    var job_files = [];
    input.preview.forEach((element) => {
        job_files.push("data/job" + hash + "/" + element);
    });

    const nii_files = job_files;

    let categories = [];
    Object.keys(nii_options).forEach(function (keys) {
        var category = nii_options[keys].options.category;
        if (!categories.includes(category)) {
            categories.push(category);
        }
    });

    const job = {
        hash: hash,
        id: input.job_id,
        created: input.created,
        pipeline_name: input.pipeline_name,
        version_number: input.pipeline_version,
        status: 4,
        hash_result: input.hash,
        age: input.age,
        sex: input.sex,
        allow_share: input.allow_share,
        options: {
            files: input.preview_options,
            categories: categories,
        },
    };

    const bounds_files = input.measures.measure.regions_bounds;
    let orderedBoundsFiles = {};
    bounds_files.forEach(file => {
        if(file.includes("female")) {
            orderedBoundsFiles.female = file;
        } else if(file.includes("male")) {
            orderedBoundsFiles.male = file;
        } else {
            orderedBoundsFiles.general = file;
        }
    });

    const page = {
        image_url: "/img/dark_logo.png",
        title: "volBrain Preview",
    };

    //console.log(nii_options);

    res.render("pages/preview.html", {
        job: job,
        nii_files: JSON.stringify(nii_files),
        nii_options: JSON.stringify(nii_options),
        page: page,
        kiosk_mode: "false",
        bounds_files: JSON.stringify(orderedBoundsFiles),
        job_qc: JSON.stringify(job_qc),
    });
});

router.post("/save-input", async function (req, res){
    const hash = req.query.hash;
    var inputPath = __dirname + "/../public/data/";
    if (hash) {
        inputPath += "job" + hash + "/";
    }
    inputPath += "input.json";

    var job_qc = JSON.parse(req.body.job_qc);
    console.log(job_qc);

    fs.readFile(inputPath).then(obj => {
        return JSON.parse(obj)
    }).then( data => {
        var input = data;
        if(JSON.stringify(input.job_qc) !== JSON.stringify(job_qc)) {
            job_qc.commentExist = true;
            input.job_qc = job_qc;
            fs.writeFile(inputPath, JSON.stringify(input, null, 2), function (err) {
                if (err) throw err;
            });
            res.status(200).send({ updated: true });
        } else {
            res.status(200).send({ updated: false });
        }
    })
    .catch(error => res.status(500).send("BAD " + error));
});

module.exports = router;