const {By} = require('selenium-webdriver');
const assert = require('assert');
const {colors, clickInElementAtPercent} = require("./utils");

/**
 * @param driver
 * @returns {Promise<void>}
 */
async function testDisplayCurve(driver) {
    const isCurveHidden = await driver.findElement(By.id('labelDiv')).getAttribute('hidden');
    const labelName = await driver.findElement(By.css('#labelName > h5')).getText();
    if(isCurveHidden) {
        console.log(colors.bg.blue, "=> no normality curve" + labelName, colors.reset);
    } else {
        const plot = await driver.findElement(By.id('myChart'));
        assert.notEqual(plot, null);
        console.log(colors.bg.blue, "=> normality curve found for " + labelName, colors.reset);
    }
}

async function changeCurrentOverlay(driver, keyPress) {
    const papayaViewer = await driver.findElement(By.id('papayaViewer0'));
    const actions = await driver.actions();
    actions.move({origin:papayaViewer}).perform();
    actions.keyDown(keyPress).perform();
}

async function displayNormalityCurve(driver) {
    try {
        await driver.findElement(By.id('papayaViewer0')).then(async (viewer) => {
            await clickInElementAtPercent(driver, viewer, 10, 50); // label with curve
            await testDisplayCurve(driver);
            await clickInElementAtPercent(driver, viewer, 10, 10); // label 0, no curve
            await testDisplayCurve(driver);

            await changeCurrentOverlay(driver, '4');

            await clickInElementAtPercent(driver, viewer, 10, 50); // label with curve
            await testDisplayCurve(driver);
            await clickInElementAtPercent(driver, viewer, 30, 30); // label 0, no curve
            await testDisplayCurve(driver);
            console.log(colors.bg.green, "Normality Curve tests passed !", colors.reset);
        });
    } catch(e){
        console.error(colors.bg.red, "Error during normality curve tests : " + e, colors.reset);
    }
}

exports.runNormalityCurveTest = async function(driver, hash) {
    await driver.get("http://localhost:5000/preview?hash=" + hash);
    //test the display of the normality curve 
    await displayNormalityCurve(driver);
}