const assert = require('assert');
const {By} = require('selenium-webdriver');
const {KEYNAME_NUMPAD, colors, sleep, clickInElementAtPercent, waitLoadingPage, loadOverlaysData} = require("./utils");

const LAYERS_categorie = "uploadedFiles";
const UPLOADED_LAYERS = ["structures_new.nii.gz"]; //, "image2_new.nii"

/**
 * Check if the buttons react correctly
 *
 * @param driver
 * @returns {Promise<void>}
 */
// TODO : A bit redundant, could be reduced in a function that takes a file to load
async function buttonsTests(driver){
    try{
        // Checks values before sending
        let sendButtonIsEnabled = await driver.findElement(By.id("buttonSendEdit")).isEnabled();
        assert.equal(sendButtonIsEnabled, false);

        let dropDownEditValue = await driver.findElement(By.id("uploadedFilesEdit")).getAttribute("value");
        assert.equal(dropDownEditValue, "uploadedFiles");

        for (let i = 0; i < UPLOADED_LAYERS.length; ++i) {
            // Load a file
            let inputFileButton = await driver.findElement(By.id("correctionFileInput"));
            inputFileButton.sendKeys(__dirname + "/resources/" + UPLOADED_LAYERS[i]);

            sendButtonIsEnabled = await driver.findElement(By.id("buttonSendEdit")).isEnabled();
            assert.equal(sendButtonIsEnabled, true);

            // Send it
            await driver.findElement(By.id("buttonSendEdit")).click();
            await sleep(1000); // sleep for wait loading

            // Checks values after sending
            let inputFileButtonValue = await driver.findElement(By.id("correctionFileInput")).getAttribute("value");
            assert.equal(inputFileButtonValue, "");
            dropDownEditValue = await driver.findElement(By.id("uploadedFilesEdit")).getAttribute("value");
            assert.equal(dropDownEditValue, UPLOADED_LAYERS[i]);
            sendButtonIsEnabled = await driver.findElement(By.id("buttonSendEdit")).isEnabled();
            assert.equal(sendButtonIsEnabled, false);
        }

        console.log(colors.bg.blue, "=> tests of the edition buttons passed", colors.reset);
    }catch(e){
        console.error(colors.bg.red, "ERROR :", colors.reset, " the tests of the edition buttons do not pass : \"", e.actual,"\" ", colors.bg.red, " != ", colors.reset, " \"", e.expected, "\"");
        console.log(e.message);
    }
}

/**
 * Test the "s" shortcut on each uploaded file
 * @param driver
 * @returns {Promise<void>}
 */
async function keyS(driver) {
    //get segmentation selected
    let fileSelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
    let fileSelectedInfo = await driver.findElement(By.id('uploadedFilesInfo')).getAttribute('value');
    const fileInitial = fileSelected;

    //get papaya viewer
    const papayaViewer = await driver.findElement(By.id('papayaViewer0'));

    // click on the left tab otherwise it doesn't work ??? I don't fucking know why :/
    await clickInElementAtPercent(driver, await driver.findElement(By.id('edits')), 99, 99);

    const actions = await driver.actions();
    //hide overlay with shorcut 's'
    actions.move({origin:papayaViewer}).perform();
    actions.keyDown('s').perform();

    fileSelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
    assert.equal(fileSelected, LAYERS_categorie);

    // For Info drop-down list
    fileSelectedInfo = await driver.findElement(By.id('uploadedFilesInfo')).getAttribute('value');
    assert.equal(fileSelectedInfo, LAYERS_categorie);

    //show again overlay with shorcut 's'
    actions.move({origin:papayaViewer}).perform();
    actions.keyDown('s').perform();
    fileSelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
    assert.equal(fileSelected, fileInitial);
    // For Info drop-down list
    fileSelectedInfo = await driver.findElement(By.id('uploadedFilesInfo')).getAttribute('value');
    assert.equal(fileSelectedInfo, fileInitial);

    console.log(colors.bg.blue, "=> shorcut \"s\" (uploaded file) passed", colors.reset);
}

/**
 * Test shortcuts 1 to 9 (keyboard + numpad) on each uploaded file
 *
 * @param driver
 * @param LAYERS The other layers (loaded when Papaya starts)
 * @returns {Promise<void>}
 */
// TODO : To be adapted in the same way as the overlays tests ?
async function keyNumbers(driver, LAYERS) {
    //get papaya viewer
    const papayaViewer = await driver.findElement(By.id('papayaViewer0'));

    const actions = await driver.actions();

    let overlaySelected, overlaySelectedInfo;
    overlaySelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
    // Hide the selected file if there is one
    if (overlaySelected !== "uploadedFiles"){
        actions.move({origin:papayaViewer}).perform();
        actions.keyDown('s').perform();
    }

    //check all shortcuts (1 to the number of layers)
    for(let keyPress = (1 + LAYERS.length); keyPress <= (LAYERS.length + UPLOADED_LAYERS.length); ++keyPress) {
        actions.move({origin:papayaViewer}).perform();
        actions.keyDown(keyPress.toString()).perform();

        overlaySelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
        overlaySelectedInfo = await driver.findElement(By.id('uploadedFilesInfo')).getAttribute('value'); // For Info drop-down list

        //show the segmentation at the position keyPress in the array of the layers
        let layerIndex = UPLOADED_LAYERS.indexOf(overlaySelected) + LAYERS.length;
        assert.equal(keyPress, layerIndex+1);
        // For Info drop-down list
        let layerIndexInfo = UPLOADED_LAYERS.indexOf(overlaySelectedInfo) + LAYERS.length;
        assert.equal(keyPress, layerIndexInfo+1);

        console.log(colors.bg.blue, "=> shorcut \"" + keyPress + "\" (uploaded file) passed", colors.reset);
    }

    //Hide the last selected file
    actions.keyDown('s').perform();

    // NumPad TEST
    for(let keyPress = (1 + LAYERS.length); keyPress <= (LAYERS.length + UPLOADED_LAYERS.length); ++keyPress) {
        actions.move({origin:papayaViewer}).perform();
        actions.keyDown(KEYNAME_NUMPAD[keyPress-1]).perform();

        overlaySelected = await driver.findElement(By.id('uploadedFilesEdit')).getAttribute('value');
        overlaySelectedInfo = await driver.findElement(By.id('uploadedFilesInfo')).getAttribute('value'); // For Info drop-down list

        // Show the file at the position keyPress in the array of the layers
        let layerIndex = UPLOADED_LAYERS.indexOf(overlaySelected) + LAYERS.length;
        assert.equal(keyPress, layerIndex+1);
        // For Info drop-down list
        let layerIndexInfo = UPLOADED_LAYERS.indexOf(overlaySelectedInfo) + LAYERS.length;
        assert.equal(keyPress, layerIndexInfo+1);

        console.log(colors.bg.blue, "=> NumPad shorcut \"" + KEYNAME_NUMPAD[keyPress-1] + "\" (uploaded file) passed", colors.reset);
    }
}

exports.runEditionTest = async function(driver, hash) {
    await driver.get("http://localhost:5000/preview?hash=" + hash);
    // Wait the loading of the last overlay
    await waitLoadingPage(driver, "../client/src/public/data", hash);

    //load layersData
    const layersData = await loadOverlaysData("../client/src/public/data", hash);
    const LAYERS = layersData[0];
    // const LAYERS_categories = layersData[1];
    // const LAYERS_enabled = layersData[2]; // not used yet

    await driver.findElement(By.linkText("Edits")).click();

    await buttonsTests(driver);

    sleep(1000);
    try{
        await driver.findElement(By.id('papayaViewer0')).then(async () => {
            // Tests for key 's' for uploaded files
            await keyS(driver);
            // Tests for key 1 to 9  for uploaded files
            await keyNumbers(driver, LAYERS);
            console.log(colors.bg.green, "Edition Overlays tests passed !", colors.reset);
        });
    } catch(e){
        console.error(colors.bg.red, "Error on shorcuts overlay (uploaded files) : " + e, colors.reset);
    }
}

