const {Builder} = require('selenium-webdriver');
require('dotenv').config();
const {runLabelsTest, runDescriptionButtonTest, runBackgroundTest} = require("./displayTests");
const {runOverlayTest} = require("./overlayTests");
const {runQCTest} = require("./qcTests");
const {runNormalityCurveTest} = require("./curveTests");
const {runArrowShortcut} = require("./arrowTests");
const { runFileField } = require('./fileField');
const {runRedirectToQC} = require('./redirectToQC');
const {runComment } = require('./commentTests');
const {runEditionTest} = require("./editionTests");


async function main() {

    const pipelineId = process.argv[2]; //get pipeline id argument
    let hash;
    switch(pipelineId) {
        case "1" :
            hash = "625e4aeb37529d8a95120c7891620dd8cd0837f5f8a71261f832a2f49105a955"; //AssemblyNet
            break;
        case "2" :
            hash = "625e4aeb37529d8a95120c7891620dd8cd0837f5f8a71261f832a2f49105a956"; //DeepLesionBrain
            break;
        default :
            hash = undefined; 
            console.log("Error : missing/invalid argument to run tests");
            console.log("Try \'npm run tests <pipeline id>\' [1 for AssemblyNet | 2 for DeepLesionBrain]");
            break;
    }

    let driver = await new Builder()
        .forBrowser(process.env['BROWSER'])
        .build();

    if(hash !== undefined) {
        await runLabelsTest(driver, hash);
        await runDescriptionButtonTest(driver, hash);
        await runOverlayTest(driver, hash);
        await runNormalityCurveTest(driver, hash);
        await runBackgroundTest(driver, hash);
        await runArrowShortcut(driver, hash);
        await runQCTest(driver, hash)
        await runRedirectToQC(driver, hash);
        await runFileField(driver, hash);
        await runComment(driver, hash);
        await runEditionTest(driver, hash);
    }

    await driver.quit();
}

main();