const {By} = require('selenium-webdriver');
const assert = require('assert');
const {loadOverlaysData, waitLoadingPage, KEYNAME_NUMPAD, colors} = require("./utils");

async function keyS(driver) {
    //get segmentation selected
    let segmentationSelected = await driver.findElement(By.id('segmentation')).getAttribute('value');
    let segmentationSelectedQC = await driver.findElement(By.id('segmentationQC')).getAttribute('value');
    const segmentationInitial = segmentationSelected;

    //get papaya viewer
    const papayaViewer = await driver.findElement(By.id('papayaViewer0'));
    
    const actions = await driver.actions();
    //hide overlay with shorcut 's'
    actions.move({origin:papayaViewer}).perform();
    actions.keyDown('s').perform();
    segmentationSelected = await driver.findElement(By.id('segmentation')).getAttribute('value');
    assert.equal(segmentationSelected, "segmentation");
    // For QC drop-down list
    segmentationSelectedQC = await driver.findElement(By.id('segmentationQC')).getAttribute('value');
    assert.equal(segmentationSelectedQC, "segmentation");
    
    //show again overlay with shorcut 's'
    actions.move({origin:papayaViewer}).perform();
    actions.keyDown('s').perform();
    segmentationSelected = await driver.findElement(By.id('segmentation')).getAttribute('value');
    assert.equal(segmentationSelected, segmentationInitial);
    // For QC drop-down list
    segmentationSelectedQC = await driver.findElement(By.id('segmentationQC')).getAttribute('value');
    assert.equal(segmentationSelectedQC, segmentationInitial);

    console.log(colors.bg.blue, "=> shorcut \"s\" passed", colors.reset);
}

async function keyNumbers(driver, LAYERS, LAYERS_categories, LAYERS_enabled) {
    //get papaya viewer
    const papayaViewer = await driver.findElement(By.id('papayaViewer0'));
    
    const actions = await driver.actions();
    
    //check if the first image/segmentation is selected or not and get the position
    let firstImage = undefined;
    let firstSegmentation = undefined;
    let firstImageIsSelected = false;
    let firstSegmentationIsSelected = false;
    let nbImage = 0;
    for(let position = 1; position <= LAYERS_categories.length; position++) {
        if(LAYERS_categories[position-1] === "image") {
            nbImage++;
            if(firstImage === undefined) {
                firstImage = position;
                if(LAYERS_enabled[position-1]) {
                    firstImageIsSelected = true;
                }
            }
        } else {
            if(firstSegmentation === undefined) {
                firstSegmentation = position;
                if(LAYERS_enabled[position-1]) {
                    firstSegmentationIsSelected = true;
                }
            }
        }
    }

    //check all shortcuts (1 to the number of layers)
    let overlaySelected, overlaySelectedQC;
    for(let keyPress = 1; keyPress <= LAYERS.length; keyPress++) {
        actions.move({origin:papayaViewer}).perform();
        actions.keyDown(keyPress.toString()).perform();

        if(LAYERS_categories[keyPress-1] === "image") {
            overlaySelected = await driver.findElement(By.id('image')).getAttribute('value'); // For Information drop-down list
            overlaySelectedQC = await driver.findElement(By.id('imageQC')).getAttribute('value'); // For QC drop-down list
            
            if(firstImageIsSelected && (keyPress === firstImage)) {
                //hide the image
                assert.equal(overlaySelected, LAYERS_categories[keyPress-1]);
                assert.equal(overlaySelectedQC, LAYERS_categories[keyPress-1]);
            } else {
                //show the image at the position keyPress in the array of the layers  
                const layerIndex = LAYERS.indexOf(overlaySelected);
                assert.equal(keyPress, layerIndex+1);
                const layerIndexQC = LAYERS.indexOf(overlaySelectedQC);
                assert.equal(keyPress, layerIndexQC+1);
            }
        } else {
            overlaySelected = await driver.findElement(By.id('segmentation')).getAttribute('value'); // For Information drop-down list
            overlaySelectedQC = await driver.findElement(By.id('segmentationQC')).getAttribute('value'); // For QC drop-down list
            
            if(firstSegmentationIsSelected && (keyPress === firstSegmentation)) {
                //hide the segmentation
                assert.equal(overlaySelected, LAYERS_categories[keyPress-1]);
                assert.equal(overlaySelectedQC, LAYERS_categories[keyPress-1]);
            } else {
                //show the segmentation at the position keyPress in the array of the layers  
                let layerIndex = LAYERS.indexOf(overlaySelected);
                assert.equal(keyPress, layerIndex+1);
                let layerIndexQC = LAYERS.indexOf(overlaySelectedQC);
                assert.equal(keyPress, layerIndexQC+1);
            }
        }
        console.log(colors.bg.blue, "=> shorcut \"" + keyPress + "\" passed", colors.reset);
    }
    // NumPad TEST
    for(let keyPress = 1; keyPress <= LAYERS.length; keyPress++) {
        actions.move({origin:papayaViewer}).perform();
        actions.keyDown(KEYNAME_NUMPAD[keyPress-1]).perform();

        if(keyPress <= nbImage) {
            //show current image 
            overlaySelected = await driver.findElement(By.id('image')).getAttribute('value');
            overlaySelectedQC = await driver.findElement(By.id('imageQC')).getAttribute('value'); // For QC drop-down list
        } else {
            overlaySelected = await driver.findElement(By.id('segmentation')).getAttribute('value');
            overlaySelectedQC = await driver.findElement(By.id('segmentationQC')).getAttribute('value'); // For QC drop-down list
        } 
        //show the segmentation at the position keyPress in the array of the layers  
        let layerIndex = LAYERS.indexOf(overlaySelected);
        assert.equal(keyPress, layerIndex+1);
        // For QC drop-down list
        let layerIndexQC = LAYERS.indexOf(overlaySelectedQC);
        assert.equal(keyPress, layerIndexQC+1);

        console.log(colors.bg.blue, "=> shorcut \"" + KEYNAME_NUMPAD[keyPress-1] + "\" passed", colors.reset);
    }
}

async function shorcutsOverlay(driver, LAYERS, LAYERS_categories, LAYERS_enabled) {
    try {
        await driver.findElement(By.id('papayaViewer0')).then(async () => {
            //tests for key 's'
            await keyS(driver);
            //tests for key 1 to 9
            await keyNumbers(driver, LAYERS, LAYERS_categories, LAYERS_enabled);
            console.log(colors.bg.green, "Overlays tests passed !", colors.reset);
        });
    } catch(e){
        console.error(colors.bg.red, "Error on shorcuts overlay : " + e, colors.reset);
    }
}



exports.runOverlayTest = async function(driver, hash) {
    
    await driver.get("http://localhost:5000/preview?hash=" + hash);
    //wait the loading of the last overlay
    await waitLoadingPage(driver, "../client/src/public/data", hash);
    //load layersData
    const layersData = await loadOverlaysData("../client/src/public/data", hash);
    const LAYERS = layersData[0];
    const LAYERS_categories = layersData[1];
    const LAYERS_enabled = layersData[2];
    //test the shorcurs
    await shorcutsOverlay(driver, LAYERS, LAYERS_categories, LAYERS_enabled);
}