const assert = require('assert');
const {By} = require('selenium-webdriver');


exports.runQCTest = async function(driver, hash) {
    try {
        await driver.get("http://localhost:5000/preview?hash=" + hash);
        
        //Tests Tabs
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.linkText("Edits")).click(); 
        await driver.findElement(By.linkText("Informations")).click(); 
        console.log(" ✅ tests Tabs passed !");
    }catch(e){
        console.error( " ❌ ERROR on Tabs tests");
    }

    try{
        //Tests question top 
        await driver.findElement(By.linkText("QC")).click(); 
        let textQuestion = await driver.findElement(By.id('questionTop')).getText().then(function(value){
            return value
        });

        assert.strictEqual(textQuestion, "How is the quality of the analysis ?")
        console.log(" ✅ tests QC Question Top passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Question tests");
    }

  

    try{
        //Tests radio Image Good
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkGoodImage')).click();

        console.log(" ✅ tests QC - Image - Good button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image Good button  tests");
    }

    try{
        //Tests radio input Medium Image
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumImage')).click();

        console.log(" ✅ tests QC - Image - Medium button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image Medium button  tests");
    }
      
    try{
        //Tests radio input Bad Image
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkBadImage')).click();

        console.log(" ✅ tests QC - Image - Bad button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image Bad button  tests");
    }

    try{
        //Tests radio input  Bad  then Good Image
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkBadImage')).click();
        await driver.findElement(By.id('checkGoodImage')).click();


        console.log(" ✅ tests QC - Image - Bad and Good button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image Bad and Good button tests");
    }

    try{
        //Tests Image - click on Medium - check Question 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumImage')).click();
        let textQuestion = await driver.findElement(By.css('#imageIssues > p')).getText().then(function(value){
            return value
        });

        assert.strictEqual(textQuestion, "What problems do the background image have?")
        console.log(" ✅ tests QC - Image - question text passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image question tests");
    }

    try{
        //Tests Image - click on Medium - ckeckbox 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumImage')).click();

        //click on checkbox to valid
        await driver.findElement(By.id('resolution')).click();
        await driver.findElement(By.id('noise')).click();
        await driver.findElement(By.id('contrast')).click();
        await driver.findElement(By.id('artefacts')).click();

        //click on checkbox to unvalid
        await driver.findElement(By.id('resolution')).click();
        await driver.findElement(By.id('noise')).click();
        await driver.findElement(By.id('contrast')).click();
        await driver.findElement(By.id('artefacts')).click();

        console.log(" ✅ tests QC - Image - checkbox passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image checkbox tests");
    }

    try{
        //Tests Image - click on Medium - text area
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.css('#imageComment')).click();
        await driver.findElement(By.css('#imageComment')).sendKeys("test text area");

        console.log(" ✅ tests QC - Image - text area passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Image text area tests");
    }


    //////////////////////////////////////////////////////////



    try{
        //Tests radio segmentation input Good 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkGoodSegmentation')).click();

        console.log(" ✅ tests QC - Segmentation - Good button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation Good button  tests");
    }


    try{
        //Tests radio segmentation input Medium 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumSegmentation')).click();

        console.log(" ✅ tests QC - Segmentation - Medium button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation Medium button  tests");
    }
      

    try{
        //Tests radio Segmentation input Bad 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkBadSegmentation')).click();

        console.log(" ✅ tests QC - Segmentation - Bad button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation Bad button  tests");
    }


    try{
        //Tests radio input  Bad  then Good
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkBadSegmentation')).click();
        await driver.findElement(By.id('checkGoodSegmentation')).click();


        console.log(" ✅ tests QC - Segmentation - Bad and Good button passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation Bad and Good button tests");
    }

    
    try{
        //Tests Input image - click on Medium - check Question 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumSegmentation')).click();
        let textQuestion = await driver.findElement(By.css('#segmentationIssues > p')).getText().then(function(value){
            return value
        });

        assert.strictEqual(textQuestion, "What problems do the segmentation have?")
        console.log(" ✅ tests QC - Segmentation - question text passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation question tests");
    }

    

    try{
        //Tests Segmentation - click on Medium - ckeckbox 
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.id('checkMediumSegmentation')).click();

        //click on checkbox to valid
        await driver.findElement(By.id('over_segmentation')).click();
        await driver.findElement(By.id('under_segmentation')).click();
        await driver.findElement(By.id('missing_structures')).click();

        //click on checkbox to unvalid
        await driver.findElement(By.id('over_segmentation')).click();
        await driver.findElement(By.id('under_segmentation')).click();
        await driver.findElement(By.id('missing_structures')).click();


        console.log(" ✅ tests QC - Segmentation - checkbox passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation checkbox tests");
    }

    
    try{
        //Tests Segmentation - click on Medium - text area
        await driver.findElement(By.linkText("QC")).click(); 
        await driver.findElement(By.css('#segmentationComment')).click();
        await driver.findElement(By.css('#segmentationComment')).sendKeys("test text area");

        console.log(" ✅ tests QC - Segmentation - text area passed !");
    }catch(e){
        console.error(" ❌ ERROR on QC Segmentation text area tests");
    }

}

